'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var join = require('path').join;
var execSync = require('child_process').execSync;
var gitFiles = require('git-files');
var Interactive = require('./interactive');

/**
 * Reads a file glob and a commit message, and commits to git.
 * @class
 */

var GitAddAndCommit = function () {
	function GitAddAndCommit() {
		var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

		_classCallCheck(this, GitAddAndCommit);

		this.options = options;

		if (options.help) {
			this.showHelpScreen();
		} else if (options.interactive) {
			new Interactive(options).run();
		} else {
			this.normal();
		}
	}

	/**
  * Attempts to add and commit. Errors out in a non-ugly way.
  */


	_createClass(GitAddAndCommit, [{
		key: 'normal',
		value: function normal() {
			var args = process.argv.slice(2);
			if (args.length < 2) {
				console.log('File glob and commit message required. See help screen for correct usage.');
				return;
			}

			var stagedFiles = gitFiles.staged('relative');
			var stageWarning = stagedFiles.length > 0;

			if (stageWarning) {
				// ignore all files that have already been staged, then stage them after this commit
				process.stdout.write('There are file(s) already staged for commit. Resetting staged files...');
				var _iteratorNormalCompletion = true;
				var _didIteratorError = false;
				var _iteratorError = undefined;

				try {
					for (var _iterator = stagedFiles[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
						var file = _step.value;

						execSync('git reset ' + file);
					}
				} catch (err) {
					_didIteratorError = true;
					_iteratorError = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion && _iterator.return) {
							_iterator.return();
						}
					} finally {
						if (_didIteratorError) {
							throw _iteratorError;
						}
					}
				}

				console.log('Done.');
			}

			try {
				var glob = args[0];
				var commitMessage = args[1];

				var files = this.getGitFilesMatching(glob);
				var _iteratorNormalCompletion2 = true;
				var _didIteratorError2 = false;
				var _iteratorError2 = undefined;

				try {
					for (var _iterator2 = files[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
						var _file2 = _step2.value;

						execSync('git add ' + _file2);
					}
				} catch (err) {
					_didIteratorError2 = true;
					_iteratorError2 = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion2 && _iterator2.return) {
							_iterator2.return();
						}
					} finally {
						if (_didIteratorError2) {
							throw _iteratorError2;
						}
					}
				}

				execSync('git commit -m "' + commitMessage + '"');
				console.log('Commit successful.');

				if (stageWarning) {
					process.stdout.write('Re-adding previously staged files...');
					var _iteratorNormalCompletion3 = true;
					var _didIteratorError3 = false;
					var _iteratorError3 = undefined;

					try {
						for (var _iterator3 = stagedFiles[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
							var _file = _step3.value;

							execSync('git add ' + _file);
						}
					} catch (err) {
						_didIteratorError3 = true;
						_iteratorError3 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion3 && _iterator3.return) {
								_iterator3.return();
							}
						} finally {
							if (_didIteratorError3) {
								throw _iteratorError3;
							}
						}
					}

					console.log('Done.');
				}
			} catch (e) {
				console.log('Encountered error -- aborting.');
				process.stdin.write('Resetting added files...');
				execSync('git reset .');
				process.stdin.write('Done.');
				console.log();
			}
		}

		/**
   * Copy-pasted from interactive.js.
   * TODO: There's probably a better way.
   */

	}, {
		key: 'getGitFilesMatching',
		value: function getGitFilesMatching(glob) {
			var files = gitFiles.all('relative');
			var regex = new RegExp(glob, this.options.caseSensitive ? '' : 'i');
			return files.filter(function (file) {
				return regex.test(file);
			});
		}

		/**
  	 * Shows the help screen.
    */

	}, {
		key: 'showHelpScreen',
		value: function showHelpScreen() {
			var helpFile = join(__dirname, '../help.txt');
			var helpScreen = execSync('cat ' + helpFile);
			console.log(helpScreen.toString('utf8'));
		}
	}]);

	return GitAddAndCommit;
}();

module.exports = GitAddAndCommit;